==> tails-iuk_0.10.orig.tar.gz: Tails-IUK-0.10/lib/Tails/IUK/UpgradeDescriptionFile/Generate.pm <==
=head1 NAME

Tails::IUK::UpgradeDescriptionFile::Generate - create and update upgrade-description files

=cut

package Tails::IUK::UpgradeDescriptionFile::Generate;

use Moose;
use Moose::Util::TypeConstraints qw{class_type};
use MooseX::Method::Signatures;
use MooseX::Types::Moose qw( :all );
use MooseX::Types::Path::Class qw{Dir File};
use MooseX::Has::Sugar::Saccharin;

with 'MooseX::Getopt::Dashes';

use 5.10.1;
use namespace::autoclean;
use warnings FATAL => 'all';

use autodie qw(:all);
use Carp::Assert;
use Digest::SHA;
use English qw{-no_match_vars};
use Path::Class;
use Tails::IUK::UpgradeDescriptionFile;


=head1 ATTRIBUTES

=cut

has previous_versions => required ro ArrayRef;
has version           => required ro Str;
has major_release     => required ro Bool;

has iso               => required coerce ro File;
has iuks              => required coerce ro Dir;
has release_checkout  => required coerce ro Dir;

has build_target      => lazy_build ro Str;
has channel           => lazy_build ro Str;
has product_name      => lazy_build ro Str;


=head1 CONSTRUCTORS AND BUILDERS

=cut

method _build_build_target  { 'i386'   }
method _build_channel       { 'stable' }
method _build_product_name  { 'Tails'  }

method BUILD {
    assert(-f $self->iso);
    assert(-d $self->release_checkout);
}


=head1 METHODS

=cut

method run {
    say STDERR q{* Creating upgrade-description file for new release },
        '(', $self->version, "): \n  ", $self->udf_for_new_release, ' ...';
    $self->create_udf_for_new_release;

    for my $previous_version (@{$self->previous_versions}) {
        say STDERR "\n", q{* Updating upgrade-description file for previous },
            'release (', $previous_version, "): \n  ",
            $self->udf_for($previous_version), ' ...';
        $self->update_udf_for_previous_release($previous_version);
    }
}

method create_udf_for_new_release () {
    my $description = Tails::IUK::UpgradeDescriptionFile->new(
        product_name    => $self->product_name,
        product_version => $self->version,
        build_target    => $self->build_target,
        channel         => $self->channel,
    );
    $self->udf_for_new_release->parent->mkpath;
    $self->udf_for_new_release->spew(
        $description->stringify
    );
}

method update_udf_for_previous_release ($previous_version) {
    my $udf         = $self->udf_for($previous_version);
    my $description = Tails::IUK::UpgradeDescriptionFile->new_from_file($udf);

    # We don't want older upgrades to be advertised forever, once we provide
    # an upgrade path to the last one.
    $description->empty_upgrades;

    $description->add_upgrade({
        version       => $self->version,
        details_url   => 'https://tails.boum.org/news/version_/'.$self->version.'/',
        type          => $self->major_release ? 'major' : 'minor',
        upgrade_paths => [$self->upgrade_paths_from_previous_release($previous_version)],
    });
    $udf->spew($description->stringify);
}

method upgrade_paths_from_previous_release ($previous_version) {
    my @paths;

    say STDERR q{  Adding full upgrade...};
    push @paths, {
        type => 'full',
        target_files => [{
            sha256 => sha256_file($self->iso),
            size   => -s $self->iso,
            url    => $self->iso_url,
        }],
    };

    if ($self->has_iuk_for($previous_version)) {
        say STDERR q{  Adding incremental upgrade...};
        my $iuk = $self->iuk_for($previous_version);
        push @paths, {
            type => 'incremental',
            target_files => [{
                sha256 => sha256_file($iuk),
                size   => -s $iuk,
                url    => $self->iuk_url($previous_version),
            }],
        };
    }
    else {
        say STDERR q{  No IUK could be found for }, $previous_version,
            q{, we will only advertise full upgrade.};
    }

    return @paths;
}

method udf_for ($version) {
    file(
        $self->release_checkout, qw{wiki src upgrade v1}, $self->product_name,
        $version, $self->build_target, $self->channel, 'upgrades.yml'
    );
}

method udf_for_new_release () {
    $self->udf_for($self->version)
}

method iso_url {
    'http://dl.amnesia.boum.org/tails/'
        . $self->channel
        . '/'
        . lc($self->product_name)
        . '-'
        . $self->build_target
        . '-'
        . $self->version
        . '/'
        . $self->product_name
        . '-'
        . $self->build_target
        . '-'
        . $self->version
        . '.iso';
}

method iuk_filename ($previous_version) {
    $self->product_name
        . '_'
        . $self->build_target
        . '_'
        . $previous_version
        . '_to_'
        . $self->version
        . '.iuk';
}

method iuk_url ($previous_version) {
    'http://dl.amnesia.boum.org/tails/'
        . $self->channel
        . '/iuk/'
        . $self->iuk_filename($previous_version);
}

method iuk_for ($previous_version) {
    file($self->iuks, $self->iuk_filename($previous_version));
}

method has_iuk_for ($previous_version) {
    -f $self->iuk_for($previous_version);
}


=head1 FUNCTIONS

=cut

sub sha256_file {
    my $file = shift;
    assert(-f $file);

    my $sha = Digest::SHA->new(256);
    $sha->addfile($file->stringify);
    return $sha->hexdigest;
}

no Moose;
1;

==> tails-iuk_1.0.orig.tar.gz: Tails-IUK-1.0/lib/Tails/IUK/UpgradeDescriptionFile/Generate.pm <==
=head1 NAME

Tails::IUK::UpgradeDescriptionFile::Generate - create and update upgrade-description files

=cut

package Tails::IUK::UpgradeDescriptionFile::Generate;

use Moose;
use Moose::Util::TypeConstraints qw{class_type};
use MooseX::Method::Signatures;
use MooseX::Types::Moose qw( :all );
use MooseX::Types::Path::Class qw{Dir File};
use MooseX::Has::Sugar::Saccharin;

with 'MooseX::Getopt::Dashes';

use 5.10.1;
use namespace::autoclean;
use warnings FATAL => 'all';

use autodie qw(:all);
use Carp::Assert;
use Digest::SHA;
use English qw{-no_match_vars};
use Path::Class;
use Tails::IUK::UpgradeDescriptionFile;


=head1 ATTRIBUTES

=cut

has previous_versions => required ro ArrayRef;
has version           => required ro Str;
has major_release     => required ro Bool;

has iso               => required coerce ro File;
has iuks              => required coerce ro Dir;
has release_checkout  => required coerce ro Dir;

has build_target      => lazy_build ro Str;
has channel           => lazy_build ro Str;
has product_name      => lazy_build ro Str;


=head1 CONSTRUCTORS AND BUILDERS

=cut

method _build_build_target  { 'i386'   }
method _build_channel       { 'stable' }
method _build_product_name  { 'Tails'  }

method BUILD {
    assert(-f $self->iso);
    assert(-d $self->release_checkout);
}


=head1 METHODS

=cut

method run {
    say STDERR q{* Creating upgrade-description file for new release },
        '(', $self->version, "): \n  ", $self->udf_for_new_release, ' ...';
    $self->create_udf_for_new_release;

    for my $previous_version (@{$self->previous_versions}) {
        say STDERR "\n", q{* Updating upgrade-description file for previous },
            'release (', $previous_version, "): \n  ",
            $self->udf_for($previous_version), ' ...';
        $self->update_udf_for_previous_release($previous_version);
    }
}

method create_udf_for_new_release () {
    my $description = Tails::IUK::UpgradeDescriptionFile->new(
        product_name    => $self->product_name,
        product_version => $self->version,
        build_target    => $self->build_target,
        channel         => $self->channel,
    );
    $self->udf_for_new_release->parent->mkpath;
    $self->udf_for_new_release->spew(
        $description->stringify
    );
}

method update_udf_for_previous_release ($previous_version) {
    my $udf         = $self->udf_for($previous_version);
    my $description = Tails::IUK::UpgradeDescriptionFile->new_from_file($udf);

    # We don't want older upgrades to be advertised forever, once we provide
    # an upgrade path to the last one.
    $description->empty_upgrades;

    $description->add_upgrade({
        version         => $self->version,
        'details-url'   => 'https://tails.boum.org/news/version_'.$self->version.'/',
        type            => $self->major_release ? 'major' : 'minor',
        'upgrade-paths' => [$self->upgrade_paths_from_previous_release($previous_version)],
    });
    $udf->spew($description->stringify);
}

method upgrade_paths_from_previous_release ($previous_version) {
    my @paths;

    say STDERR q{  Adding full upgrade...};
    push @paths, {
        type           => 'full',
        'target-files' => [{
            sha256 => sha256_file($self->iso),
            size   => -s $self->iso,
            url    => $self->iso_url,
        }],
    };

    if ($self->has_iuk_for($previous_version)) {
        say STDERR q{  Adding incremental upgrade...};
        my $iuk = $self->iuk_for($previous_version);
        push @paths, {
            type           => 'incremental',
            'target-files' => [{
                sha256 => sha256_file($iuk),
                size   => -s $iuk,
                url    => $self->iuk_url($previous_version),
            }],
        };
    }
    else {
        say STDERR q{  No IUK could be found for }, $previous_version,
            q{, we will only advertise full upgrade.};
    }

    return @paths;
}

method udf_for ($version) {
    file(
        $self->release_checkout, qw{wiki src upgrade v1}, $self->product_name,
        $version, $self->build_target, $self->channel, 'upgrades.yml'
    );
}

method udf_for_new_release () {
    $self->udf_for($self->version)
}

method iso_url {
    'http://dl.amnesia.boum.org/tails/'
        . $self->channel
        . '/'
        . lc($self->product_name)
        . '-'
        . $self->build_target
        . '-'
        . $self->version
        . '/'
        . $self->product_name
        . '-'
        . $self->build_target
        . '-'
        . $self->version
        . '.iso';
}

method iuk_filename ($previous_version) {
    $self->product_name
        . '_'
        . $self->build_target
        . '_'
        . $previous_version
        . '_to_'
        . $self->version
        . '.iuk';
}

method iuk_url ($previous_version) {
    'http://dl.amnesia.boum.org/tails/'
        . $self->channel
        . '/iuk/'
        . $self->iuk_filename($previous_version);
}

method iuk_for ($previous_version) {
    file($self->iuks, $self->iuk_filename($previous_version));
}

method has_iuk_for ($previous_version) {
    -f $self->iuk_for($previous_version);
}


=head1 FUNCTIONS

=cut

sub sha256_file {
    my $file = shift;
    assert(-f $file);

    my $sha = Digest::SHA->new(256);
    $sha->addfile($file->stringify);
    return $sha->hexdigest;
}

no Moose;
1;
